#ifndef PSYNCCONFIG_H
#define PSYNCCONFIG_H

#include <QDialog>
#include <QMouseEvent>
#include <QKeyEvent>
#include <QFileDialog>
#include <QDir>
#include <QLineEdit>
#include <QSocketNotifier>
#include <QProcess>
#include <QDateTime>
#include <QMessageBox>
#include <QCursor>
#include <kuser.h>
#include <klocale.h>
#include <kglobal.h>
#include "framelogin.h"
#include "ipcpacket.h"
#include "ipc_interface.h"
#include "unixsocket.h"
#include "libcfg.h"

namespace Ui {
    class PSyncConfig;    
}

class PSyncConfig : public QDialog
{
    Q_OBJECT

public:
    explicit PSyncConfig(QWidget *parent = 0);
    ~PSyncConfig();    

protected:
    bool eventFilter(QObject *obj, QEvent *ev);

private slots:
    void syncIntervalsChanged(int index);
    void setLineEditText(int index);
    void setComboBoxEnable(int state);
    void callFileDialog();
    void openAccountInBrowser();
    void syncronizeStartStop();
    void receiveNotification();
    void requestPassword();
    void settingsChanged();
    void sendSyncronizeIs();    

private:
    inline void initComboBox();
    inline void createComboBoxsLineEdit();
    inline void readSettings();        
    inline void saveSettings();
    inline void syncSettingsInstance();
    inline void initSocket();
    inline void initSyncranizationState();
    inline void setUserPixmap();
    inline void setLastSyncNote(const QDateTime &lastsync = QDateTime());
    inline void check_request_lastsync_from_config();
    inline void initFrameLogin();
    inline void initFrames();

    void setState(unsigned char* comm, unsigned long len);
    void setPasswordWindow();
    void authAnswer(unsigned char *result);
    void setQuoteTotal(unsigned long long bytes);
    void setQuoteUsed(unsigned long long bytes);
    void initMargins();
    void sendSettingsChangedNotify();    
    unsigned long fromIndexToSec(int index);
    int fromSecToIndex(unsigned long Sec);

private:
    inline void setUiComponentsAttr(bool sync_settings, const QString &button_text);

    Ui::PSyncConfig *ui;    
    QLineEdit *m_ln;
    QLineEdit *m_svpnt;
    QPushButton *m_login_button;
    QPushButton *m_create_account_button;
    FrameLogin *frmlgn;   

    bool m_sync_is_running;
    bool m_request_lastsync_from_config;

    UnixSocket *m_us;
    QSocketNotifier *m_sn;
    Qt::WindowFlags wf;

    QDateTime m_datetime_lastsync;

    QString m_login;
    QString m_password;

    QString m_pathname_file_config_catalog;
    QString m_pathname_file_info_catalog;

private:        
    static const char* CONFIG_FILE;
    static const char* CONFIG_FILE_LASTSYNC;    
    static const char* DEFAULT_FOLDER_STR_SEPARATOR;

    static const char* SYNC_IS_SETTING;
    static const char* INTERVAL_SETTING;
    static const char* DEFAULT_FOLDER_SETTING;
    static const char* LAST_TIME_SYNCRONIZATION_SETTING;
    static const char* LOGIN_SETTING;
    static const char* PASSWORD_SETTING;
    static const char* QUOTA_USED_SETTINGS;
    static const char* QUOTA_TOTAL_SETTINGS;
};

#endif // PSYNCCONFIG_H
