#include "psyncconfig.h"
#include "ui_psyncconfig.h"

const char* PSyncConfig::CONFIG_FILE = "config";
const char* PSyncConfig::CONFIG_FILE_LASTSYNC = "config_lastsync";
const char* PSyncConfig::DEFAULT_FOLDER_STR_SEPARATOR = ": ";

const char* PSyncConfig::SYNC_IS_SETTING = "sync_is";
const char* PSyncConfig::INTERVAL_SETTING = "interval";
const char* PSyncConfig::DEFAULT_FOLDER_SETTING = "default_folder";
const char* PSyncConfig::LAST_TIME_SYNCRONIZATION_SETTING = "config_lastsync";
const char* PSyncConfig::LOGIN_SETTING = "login";
const char* PSyncConfig::PASSWORD_SETTING = "password";
const char* PSyncConfig::QUOTA_USED_SETTINGS = "quota_used";
const char* PSyncConfig::QUOTA_TOTAL_SETTINGS = "quota_total";

PSyncConfig::PSyncConfig(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::PSyncConfig)
{
    ui->setupUi(this);

    initFrames();
    initFrameLogin();

    initComboBox();
    createComboBoxsLineEdit();

    ui->comboBox_syncronize_interval->installEventFilter(this);
    connect(ui->checkBox_is_syncronized, SIGNAL(stateChanged(int)), this, SLOT(setComboBoxEnable(int)));
    connect(ui->pushButton_change_folder, SIGNAL(clicked()), this, SLOT(callFileDialog()));    
    connect(ui->pushButton_parameters, SIGNAL(clicked()), this, SLOT(openAccountInBrowser()));

    syncSettingsInstance();
    readSettings();

    setLineEditText(ui->comboBox_syncronize_interval->currentIndex());
    setUserPixmap();

    initSocket();
    connect(ui->pushButton_syncronize, SIGNAL(clicked()), this, SLOT(syncronizeStartStop()) );

    initSyncranizationState();
    initMargins();

    connect(ui->checkBox_is_syncronized, SIGNAL(stateChanged(int)), this, SLOT(settingsChanged()));
    connect(ui->checkBox_is_syncronized, SIGNAL(stateChanged(int)), this, SLOT(sendSyncronizeIs()));
    connect(ui->comboBox_syncronize_interval, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsChanged()));
    
    
    ui->pushButton_save->setVisible(false);
    ui->pushButton_restore->setVisible(false);
    ui->pushButton_clear->setVisible(false);
    ui->pushButton_new_save_point->setVisible(false);
    ui->comboBox_save_point->setVisible(false);
    ui->label_save_date->setVisible(false);
    ui->label_save_date_note->setVisible(false);
    ui->label_save_point_note->setVisible(false);
    ui->label_size->setVisible(false);
    ui->label_size_notr->setVisible(false);
    ui->label_new_save_point_note->setVisible(false);
    ui->label_name_save_point->setVisible(false);
    ui->lineEdit_name_save_point->setVisible(false);
    ui->line->setVisible(false);
}

PSyncConfig::~PSyncConfig()
{
    delete ui;

    saveSettings();    

    m_us->finish();
    delete m_us;
}

void PSyncConfig::initFrames()
{
    ui->frame_save_point->setFrameStyle(QFrame::Plain);
    ui->frame_settings->setFrameStyle(QFrame::Plain);
}

void PSyncConfig::initFrameLogin()
{
    frmlgn = new FrameLogin(ui->tab_account);
    frmlgn->setFrameStyle(QFrame::Plain);        
    frmlgn->setVisible(false);    

    ui->tab_account->layout()->addWidget(frmlgn);
    m_login_button = frmlgn->getButtonLogin();
    m_create_account_button = frmlgn->getButtonCreateAccount();
    m_create_account_button->setAutoFillBackground(true);
    m_create_account_button->setStyleSheet("  border: none; color: blue; ");

    connect(m_login_button, SIGNAL(clicked()), this, SLOT(requestPassword()));
    connect(m_create_account_button, SIGNAL(clicked()), this, SLOT(openAccountInBrowser()));
}

void PSyncConfig::syncIntervalsChanged(int index)
{    
    setLineEditText(index);
}

void PSyncConfig::setLineEditText(int index)
{        
    m_ln->setText(ui->comboBox_syncronize_interval->itemText(index));
}

void PSyncConfig::initComboBox()
{
    ui->comboBox_syncronize_interval->insertSeparator(ui->comboBox_syncronize_interval->count() - 1);
    connect(ui->comboBox_syncronize_interval, SIGNAL(activated(int)), this, SLOT(syncIntervalsChanged(int)));
    connect(ui->comboBox_syncronize_interval, SIGNAL(highlighted(int)), this, SLOT(setLineEditText(int)));
    ui->comboBox_syncronize_interval->setInsertPolicy(QComboBox::NoInsert);        
}

void PSyncConfig::createComboBoxsLineEdit()
{
    m_ln = new QLineEdit();
    m_ln->setReadOnly(true);
    ui->comboBox_syncronize_interval->setLineEdit(m_ln);

    m_svpnt = new QLineEdit;
    m_svpnt->setReadOnly(true);
    ui->comboBox_save_point->setLineEdit(m_svpnt);
}

bool PSyncConfig::eventFilter(QObject *obj, QEvent *event)
{
    if ((obj == ui->comboBox_syncronize_interval) && (event->type() == QEvent::KeyPress))
    {
        QKeyEvent *ke = (QKeyEvent*)event;
        if (ke->key() == Qt::Key_Space)
        {
            QMouseEvent *me = new QMouseEvent(QEvent::MouseButtonPress, QPoint(ui->comboBox_syncronize_interval->width() - 5, ui->comboBox_syncronize_interval->height() - 5), Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
            QCoreApplication::postEvent(ui->comboBox_syncronize_interval, me);
        }
    }    

    return QDialog::eventFilter(obj, event);
}

void PSyncConfig::setComboBoxEnable(int state)
{
    qDebug("state = %d", state);
    ui->comboBox_syncronize_interval->setEnabled(state);    
    ui->pushButton_syncronize->setEnabled(state);
}

void PSyncConfig::readSettings()
{    
    int size_value = 512;
    char value[size_value];
    bool correct = true;
    bool correct_quota = true;

    if (get_param_str(m_pathname_file_config_catalog.toLocal8Bit().data(), SYNC_IS_SETTING, value, size_value) == LIBCFG_SUCCESS)
        ui->checkBox_is_syncronized->setCheckState((Qt::CheckState)QVariant(value).toInt());
    else
        correct = false;

    if (ui->checkBox_is_syncronized->checkState())
    {
        unsigned long interval;
        if (get_param_ul(m_pathname_file_config_catalog.toLocal8Bit().data(), INTERVAL_SETTING, &interval) == LIBCFG_SUCCESS)
            ui->comboBox_syncronize_interval->setCurrentIndex(fromSecToIndex(interval));
        else
            correct = false;
    }

    QString default_folder;
    if (get_param_str(m_pathname_file_config_catalog.toLocal8Bit().data(), DEFAULT_FOLDER_SETTING, value, size_value) == LIBCFG_SUCCESS)
    {
        if (value[0] == ' ')
            default_folder = "";
        else
            default_folder = QVariant(value).toString();
    }
    else
        correct = false;


    if (default_folder.isEmpty())
        ui->label_current_folder->setText(ui->label_current_folder->text() + QDir::homePath());
    else
        ui->label_current_folder->setText(ui->label_current_folder->text() + default_folder);

    if (get_param_str(m_pathname_file_config_catalog.toLocal8Bit().data(), LAST_TIME_SYNCRONIZATION_SETTING, value, size_value) == LIBCFG_SUCCESS)
    {
        if (value[0] != ' ')
            setLastSyncNote(QVariant(value).toDateTime());
    }
    else
        correct = false;

    unsigned long long quota;
    unsigned long long quota_u;
    if (get_param_ull(m_pathname_file_info_catalog.toLocal8Bit().data(), QUOTA_TOTAL_SETTINGS, &quota) == LIBCFG_SUCCESS)
    {
        setQuoteTotal(quota);
    }
    else
        correct_quota = false;    

    if (get_param_ull(m_pathname_file_info_catalog.toLocal8Bit().data(), QUOTA_USED_SETTINGS, &quota_u) == LIBCFG_SUCCESS)
    {
        setQuoteUsed(quota_u);
    }
    else
        correct_quota = false;    

    if (correct_quota)
        ui->progressBar->setToolTip(i18n("Used %1 Mb. of available %2 Mb.").arg(quota_u / 1024 / 1024).arg(quota / 1024 / 1024));

    if (get_param_str(m_pathname_file_config_catalog.toLocal8Bit().data(), LOGIN_SETTING, value, size_value) == LIBCFG_SUCCESS)
    {
	m_login = value;
	ui->label_account->setText(value);
	frmlgn->setLogin(value);
    }
    else
    {
        setPasswordWindow();
        return;
    }

    if (get_param_str(m_pathname_file_config_catalog.toLocal8Bit().data(), PASSWORD_SETTING, value, size_value) == LIBCFG_SUCCESS)
        m_password = QVariant(value).toString();
    else
    {        
        setPasswordWindow();
        return;
    }

    if (!correct)
        QMessageBox::critical(this, QString("Sync Client"), i18n("Error reading configuration"), QMessageBox::Ok);

    if (m_password.isEmpty())
        setPasswordWindow();
}

void PSyncConfig::saveSettings()
{    
    if (set_param_str(m_pathname_file_config_catalog.toLocal8Bit().data(), SYNC_IS_SETTING, QVariant(ui->checkBox_is_syncronized->checkState()).toString().toLocal8Bit().data()) != LIBCFG_SUCCESS)
    {
        QMessageBox::critical(this, QString("Sync Client 1"), i18n("Error saving settings"), QMessageBox::Ok);
        return;
    }

    unsigned long interval = fromIndexToSec(ui->comboBox_syncronize_interval->currentIndex());
    if (set_param_ul(m_pathname_file_config_catalog.toLocal8Bit().data(), INTERVAL_SETTING, interval) != LIBCFG_SUCCESS)
    {
        QMessageBox::critical(this, QString("Sync Client 2"), i18n("Error saving settings"), QMessageBox::Ok);
        return;
    }

    QStringList sl = ui->label_current_folder->text().split(QString(DEFAULT_FOLDER_STR_SEPARATOR));
    if (set_param_str(m_pathname_file_config_catalog.toLocal8Bit().data(), DEFAULT_FOLDER_SETTING, QVariant(sl[1]).toString().toLocal8Bit().data()) != LIBCFG_SUCCESS)
    {
        QMessageBox::critical(this, QString("Sync Client 3"), i18n("Error saving settings"), QMessageBox::Ok);
        return;
    }

    if (!m_login.isEmpty())
        if (set_param_str(m_pathname_file_config_catalog.toLocal8Bit().data(), LOGIN_SETTING, QVariant(m_login).toString().toLocal8Bit().data()) != LIBCFG_SUCCESS)
        {
            QMessageBox::critical(this, QString("Sync Client 4"), i18n("Error saving settings"), QMessageBox::Ok);
            return;
        }

    if (!m_password.isEmpty())
        if (set_param_str(m_pathname_file_config_catalog.toLocal8Bit().data(), PASSWORD_SETTING, QVariant(m_password).toString().toLocal8Bit().data()) != LIBCFG_SUCCESS)
        {
            QMessageBox::critical(this, QString("Sync Client 5"), i18n("Error saving settings"), QMessageBox::Ok);
            return;
        }

    sendSettingsChangedNotify();
}

void PSyncConfig::setPasswordWindow()
{
    ui->frame_save_point->setVisible(false);
    frmlgn->show();
}

void PSyncConfig::syncSettingsInstance()
{
    m_pathname_file_config_catalog = QString("%1%2%3").arg(QDir::homePath()).arg(PROGRAM_CATALOG).arg(SETTINGS_CATALOG);
    m_pathname_file_info_catalog = QString("%1%2%3").arg(QDir::homePath()).arg(PROGRAM_CATALOG).arg(INFO_CATALOG);
}

void PSyncConfig::callFileDialog()
{
    QStringList pe = QProcess::systemEnvironment();

    QStringList sl = ui->label_current_folder->text().split(QString(DEFAULT_FOLDER_STR_SEPARATOR));

    QString sf = QFileDialog::getExistingDirectory(this, i18n("Folder to save files by default"), sl[1], QFileDialog::ShowDirsOnly);

    if (!sf.isEmpty())
    {
        ui->label_current_folder->setText(sl[0] + QString(": ") + sf);
        this->saveSettings();
    }
}

void PSyncConfig::openAccountInBrowser()
{
    QString program = "/usr/bin/kioclient";
    QStringList arguments;
    arguments << "exec" << "http://sync.2safe.com";
    QProcess* process = new QProcess(this);
    process->start(program, arguments);
}

void PSyncConfig::syncronizeStartStop()
{
    const char *data;
    QString button_text;

    bool is_enabled = false;

    if (m_sync_is_running)
    {
        is_enabled = true;

        data = STOP_SYNCRONIZATION;
        button_text = i18n("Synchronize now");
        setLastSyncNote();
    }
    else
    {
        data = START_SYNCRONIZATION;
        button_text = i18n("Undo synchronization");
    }

    IPCPacket* pkt = IPCPacket::Create(SYNCRONIZATION_STATUS, (char*)CONFIG_PROGRAM_NAME, (unsigned char*)data, (unsigned long)strlen(data));

    QString receiver_address(QDir::homePath() + (const char*)PROGRAM_CATALOG + (const char*)SYNCRONIZE_PROGRAM_NAME);
    if (m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), receiver_address.toLocal8Bit().data()) < 0)
    {
        qDebug("Error of writeDatagram");
        QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);

        if (pkt)
            delete pkt;

        return;
    }

    receiver_address = QDir::homePath() + (const char*)PROGRAM_CATALOG + (const char*)NOTIFY_PROGRAM_NAME;
    if (m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), receiver_address.toLocal8Bit().data()) < 0)
    {
        qDebug("Error of writeDatagram");
        QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
    }

    /*(ui->pushButton_syncronize->setText(button_text);

    if (ui->pushButton_syncronize->isEnabled())
    {
        ui->checkBox_is_syncronized->setEnabled(is_enabled);
        ui->comboBox_syncronize_interval->setEnabled(is_enabled);
    }*/

    m_sync_is_running = !m_sync_is_running;
    setUiComponentsAttr(!m_sync_is_running, button_text);

    if (pkt)
        delete pkt;
}

void PSyncConfig::initSocket()
{
    m_us = new UnixSocket();
    if (m_us->init())
    {
        qDebug("sockfd = %d", m_us->getSockfd());
        QString self_address(QDir::homePath() + (const char*)PROGRAM_CATALOG + (const char*)CONFIG_PROGRAM_NAME);
        QDir dir(QDir::homePath() + (const char*)PROGRAM_CATALOG);
        if (!dir.exists())
            dir.mkdir(QDir::homePath() + (const char*)PROGRAM_CATALOG);

        if (m_us->bind(self_address.toLocal8Bit().data()))
        {
            m_sn = new QSocketNotifier(m_us->getSockfd(), QSocketNotifier::Read);
            connect(m_sn , SIGNAL(activated(int )), this, SLOT(receiveNotification()));
        }
        else
        {
            qDebug("Error of local socket bind");
            QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
        }
    }
    else
    {
        qDebug("Error of local socket init");
        QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
    }

    m_sync_is_running = false;    
}

void PSyncConfig::receiveNotification()
{    
    IPCPacket *m_pkt = NULL;
    int total_bytes = m_us->bytesAvailable();
    if (m_pkt)
        return;

    m_pkt = new IPCPacket();
    m_us->readDatagram((char*)m_pkt->readBuffer(total_bytes), total_bytes);    

    if (m_pkt->IsValid())
    {
        switch (m_pkt->getType())
        {
            case SYNCRONIZATION_STATUS:
                qDebug("data = %s, len = %d", m_pkt->getData(), m_pkt->getDataLen());
                setState(m_pkt->getData(), m_pkt->getDataLen());
                break;
            case ACTIVATE_COMMAND:
                raise();
                activateWindow();
            case SETTINGS_CLOSE_COMMAND:
                close();
                break;
            case ACCOUNT_VALID:
                authAnswer(m_pkt->getData());
                break;
            case PASSWORD_CHANGED:
                m_password.clear();
                setPasswordWindow();
                break;
            default:
                qDebug("Unknown type of message. The number of message is %d", (int)m_pkt->getType());
        }
    }

    if (m_pkt)
        delete m_pkt;
}

void PSyncConfig::setState(unsigned char* comm, unsigned long len)
{
    unsigned char data[len + 1];
    memcpy(data, comm, len);
    data[len] = '\0';

    QString button_text;    

    if (strcmp(START_SYNCRONIZATION, (const char*)data) == 0)
    {
        m_sync_is_running = true;
        button_text = i18n("Undo synchronization");
    }
    else
    {        
        m_sync_is_running = false;
        button_text = i18n("Synchronize now");

        check_request_lastsync_from_config();
    }

    setUiComponentsAttr(!m_sync_is_running, button_text);    
}

void PSyncConfig::check_request_lastsync_from_config()
{
    if (m_request_lastsync_from_config)
        setLastSyncNote();
    else
        m_request_lastsync_from_config = true;
}

void PSyncConfig::initSyncranizationState()
{    
    const char *data = (const char*)"1";    //Ничего незначащие данные, в данном случае смысловую нагрузку несёт только тип сообщения
    IPCPacket* pkt = IPCPacket::Create(CURRENT_SYNC_STATUS, (char*)CONFIG_PROGRAM_NAME, (unsigned char*)data, (unsigned long)strlen(data));

    QString receiver_address(QDir::homePath() + (const char*)PROGRAM_CATALOG + (const char*)NOTIFY_PROGRAM_NAME);
    if (m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), receiver_address.toLocal8Bit().data()) < 0)
    {
        qDebug("Error of writeDatagram");
        QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
    }

    if (pkt)
        delete pkt;

    m_request_lastsync_from_config = false;
}

void PSyncConfig::setUiComponentsAttr(bool sync_settings, const QString &button_text)
{
    ui->pushButton_syncronize->setText(button_text);   

    if (button_text == i18n("Synchronize now"))
    {

        ui->checkBox_is_syncronized->setEnabled(true);
        if (ui->pushButton_syncronize->isEnabled())
            ui->comboBox_syncronize_interval->setEnabled(true);
    }
    else
    {
        ui->checkBox_is_syncronized->setEnabled(false);
        ui->comboBox_syncronize_interval->setEnabled(false);
    }

    /*if (sync_settings)
    {
        ui->checkBox_is_syncronized->setEnabled(true);                

        ui->comboBox_syncronize_interval->setEnabled(true);
        ui->pushButton_syncronize->setEnabled(true);
        setLineEditText(ui->comboBox_syncronize_intsetUiComponentsAttrerval->count() - 1);
    }*/
}

void PSyncConfig::setUserPixmap()
{
    KUser *user = new KUser();
    QString user_avatar_path = user->faceIconPath();
    qDebug("path = %s", user_avatar_path.toLocal8Bit().data());

    if (!user_avatar_path.isEmpty())
        ui->label_image->setPixmap(QPixmap(user->faceIconPath()));
}

void PSyncConfig::setLastSyncNote(const QDateTime &lastsync)
{
    KLocale *aLocale = KGlobal::locale();

    if (lastsync.isNull())    
        ui->label_last_syncronization->setText(i18n("The last synchronization: ") + aLocale->formatDateTime(QDateTime::currentDateTime(), KLocale::FancyShortDate).toLower());    
    else
        ui->label_last_syncronization->setText(i18n("The last synchronization: ") + aLocale->formatDateTime(lastsync, KLocale::FancyShortDate).toLower());
}

void PSyncConfig::requestPassword()
{
    QString login = frmlgn->getLogin();
    QString password = frmlgn->getPassword();
    qDebug("login = %s, password = %s", login.toLocal8Bit().data(), password.toLocal8Bit().data());

    if ((login.isEmpty()) || (password.isEmpty()))
    {
        QMessageBox::warning(this, i18n("Sync Client"), i18n("Need to enter your login and password of nonzero length"), QMessageBox::Ok);
        return ;
    }

    QString info = login + ":" + password;
    qDebug("info = %s", info.toLocal8Bit().data());
    int info_len = info.length();
    char data[info_len];
    memcpy(data, info.toLocal8Bit().data(), info_len);
    qDebug("data = %s, info = %d", data, info_len);
    for (int i = 0; i < info_len; i++)
        qDebug("data[%d] = %d", i, data[i]);

    IPCPacket* pkt = IPCPacket::Create(REQUEST_LOGIN_AND_PASSWORD, (char*)CONFIG_PROGRAM_NAME, (unsigned char*)data, (unsigned long)info_len);

    QString receiver_address(QDir::homePath() + (const char*)PROGRAM_CATALOG + (const char*)SYNCRONIZE_PROGRAM_NAME);
    if (m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), receiver_address.toLocal8Bit().data()) < 0)
    {
        qDebug("Error of writeDatagram");
        QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
    }

    if (pkt)
        delete pkt;

    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
}

void PSyncConfig::authAnswer(unsigned char *result)
{
    QApplication::restoreOverrideCursor();


    if (*result == '1')
    {
        frmlgn->setVisible(false);
        ui->frame_save_point->setVisible(true);

        m_login = frmlgn->getLogin();
        m_password = frmlgn->getPassword();

// FIX IT

    m_login = frmlgn->getLogin();
    ui->label_account->setText(m_login);
    
    
    unsigned long long quota;
    unsigned long long quota_u;
    bool correct_quota = true;    
    if (get_param_ull(m_pathname_file_info_catalog.toLocal8Bit().data(), QUOTA_TOTAL_SETTINGS, &quota) == LIBCFG_SUCCESS)
    {
        setQuoteTotal(quota);
    }
    else
        correct_quota = false;    

    if (get_param_ull(m_pathname_file_info_catalog.toLocal8Bit().data(), QUOTA_USED_SETTINGS, &quota_u) == LIBCFG_SUCCESS)
    {
        setQuoteUsed(quota_u);
    }
    else
        correct_quota = false;    

    if (correct_quota)
        ui->progressBar->setToolTip(i18n("Used %1 Mb. of available %2 Mb.").arg(quota_u / 1024 / 1024).arg(quota / 1024 / 1024));
        

    }
    else
    {
        QMessageBox::critical(this, i18n("Sync Client"), i18n("Account name or password is invalid"), QMessageBox::Ok);
    }
}


void PSyncConfig::setQuoteTotal(unsigned long long bytes)
{
    unsigned long long full = bytes / 1024 / 1024;
    unsigned long long half = full / 2;

    ui->progressBar->setMaximum(full);
    ui->label_full->setText(QString::number(full) + i18n(" Mb."));
    ui->label_half->setText(QString::number(half) + i18n(" Mb."));
}

void PSyncConfig::setQuoteUsed(unsigned long long bytes)
{
    unsigned int value = bytes / 1024 / 1024;
    ui->progressBar->setValue(value);
}

void PSyncConfig::initMargins()
{
    QString padding = "padding: 5px 10px 5px 10px;";

    ui->pushButton_save->setStyleSheet(padding);
    ui->pushButton_restore->setStyleSheet(padding);
    ui->pushButton_clear->setStyleSheet(padding);
    ui->pushButton_parameters->setStyleSheet(padding);
    ui->pushButton_new_save_point->setStyleSheet(padding);
    ui->pushButton_syncronize->setStyleSheet(padding);
    ui->pushButton_change_folder->setStyleSheet(padding);
}

void PSyncConfig::sendSettingsChangedNotify()
{    
    const char *data = "1"; //Ничего не значащие данные
    IPCPacket* pkt = IPCPacket::Create(SETTINGS_CHANGED, (char*)CONFIG_PROGRAM_NAME, (unsigned char*)data, (unsigned long)strlen(data));

    QString receiver_address(QDir::homePath() + (const char*)PROGRAM_CATALOG + (const char*)SYNCRONIZE_PROGRAM_NAME);
    if (m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), receiver_address.toLocal8Bit().data()) < 0)
    {
        qDebug("Error of writeDatagram");
        QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
    }

    if (pkt)
        delete pkt;
}

void PSyncConfig::settingsChanged()
{
    saveSettings();
}

void PSyncConfig::sendSyncronizeIs()
{
    qDebug("sendSyncronizeIs");
    const char *data;
    if (ui->checkBox_is_syncronized->checkState() == Qt::Checked)
        data = "1";
    else
        data = "0";

    IPCPacket* pkt = IPCPacket::Create(SYNCRONIZE_IS, (char*)CONFIG_PROGRAM_NAME, (unsigned char*)data, (unsigned long)strlen(data));

    QString receiver_address(QDir::homePath() + (const char*)PROGRAM_CATALOG + (const char*)NOTIFY_PROGRAM_NAME);
    qDebug("receiver_address = %s", receiver_address.toLocal8Bit().data());
    if (m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), receiver_address.toLocal8Bit().data()) < 0)
    {
        qDebug("Error of writeDatagram !!!");
        QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
    }

    if (pkt)
        delete pkt;
}


unsigned long PSyncConfig::fromIndexToSec(int index)
{
    if (index == 0)
        return 900;

    if (index == 1)
        return 1800;

    if (index == 2)
        return 3600;

    return 0;
}

int PSyncConfig::fromSecToIndex(unsigned long Sec)
{
    if (Sec == 900)
        return 0;

    if (Sec == 1800)
        return 1;

    if (Sec == 3600)
        return 2;

    return 4;
}


