#ifndef PSYNCNOTIFY_H
#define PSYNCNOTIFY_H

#include <QtGui>
#include <QFile>
#include <QSocketNotifier>
#include <QMessageBox>
#include <QMovie>
#include <kprocess.h>
#include <knotification.h>
#include <klocale.h>
#include <kglobal.h>
#include <kapplication.h>
#include "ipc_interface.h"
#include "ipcpacket.h"
#include "unixsocket.h"
#include "libcfg.h"

class PSyncNotify : public QDialog
{
    Q_OBJECT

public:
    PSyncNotify(KApplication *app, QWidget *parent = 0);
    ~PSyncNotify();        

private slots:
    void receiveNotification();
    void iconActivated(QSystemTrayIcon::ActivationReason reason);    
    void syncronizeStartStop();
    void openSettings();
    void closeApp();
    void updateIcon();

private:
    void createActions();
    void createTrayIcon();
    QLabel *createLabel(QWidget *widget, const QString &text, Qt::AlignmentFlag aligment, int margin);
    QFont createFont(const QString &name, int size, bool is_bold, bool is_italic);

    inline void initSocket();
    inline void initProcessManager();
    inline void readSettings();
    inline void saveSettings();
    inline void syncSettingsInstance();
    inline void setLastSyncNote(const QDateTime &lastsync = QDateTime());
    inline void initSyncranizationState();
    inline void check_request_lastsync_from_config();

    void setState(unsigned char* comm, unsigned long len);
    void startSettingsProcess();
    void sendCommandToSettingsWindow(int type_command);
    void checkStateStatus();
    void setSyncronizeEnable(unsigned char* comm, unsigned long len);
    void setIconSync();

private:
    QWidgetAction *nameprogramAction;
    QWidgetAction *lastsynctimeAction;
    QAction *syncrAction;
    QAction *opensettingsAction;    

    QSystemTrayIcon *trayIcon;
    QMenu *trayIconMenu;
    QIcon *m_icon;

    UnixSocket *m_us;
    QSocketNotifier *m_sn;

    bool m_sync_is_running;

    KProcess *m_process;
    QString m_program;

    bool m_request_lastsync_from_sync;

    QString m_pathname_file_config;

    KApplication *m_app;

    QDateTime m_datetime_lastsync;

    QMovie *m_gif;
private:            
    static const char* CONFIG_FILE_LASTSYNC;
    static const char* LAST_TIME_SYNCRONIZATION_SETTING;
    static const char* SETTINGS_PROGRAM_PATH;

    static const char* TITLE;
    static const int TIMEOUT;
    static const char* DEFAULT_FOLDER_STR_SEPARATOR;
};

#endif // PSYNCNOTIFY_H
