#include "psyncnotify.h"

const char* PSyncNotify::CONFIG_FILE_LASTSYNC = "config_lastsync";
const char* PSyncNotify::LAST_TIME_SYNCRONIZATION_SETTING = "last_time_syncronization";
const char* PSyncNotify::SETTINGS_PROGRAM_PATH = "psyncconfig";

const char* PSyncNotify::TITLE = "PowerSync Client";
const int PSyncNotify::TIMEOUT = 5000;
const char* PSyncNotify::DEFAULT_FOLDER_STR_SEPARATOR = ": ";

PSyncNotify::PSyncNotify(KApplication *app,QWidget *parent)
    : QDialog(parent)
{
    m_icon = new QIcon(QString(":/red_status.png"));
    setWindowIcon(*m_icon);

    createActions();
    createTrayIcon();

    trayIcon->show();
    trayIcon->setToolTip(i18n("Client PowerSync") );

    initSocket();    
    initProcessManager();

    syncSettingsInstance();
    readSettings();    

    initSyncranizationState();
    m_app = app;       

    m_gif = new QMovie(":animation.gif");
    connect(m_gif, SIGNAL(frameChanged(int)), this, SLOT(updateIcon()));
}

PSyncNotify::~PSyncNotify()
{
    sendCommandToSettingsWindow(SETTINGS_CLOSE_COMMAND);
    m_us->finish();
    delete m_us;

    saveSettings();    
}

void PSyncNotify::createActions()
{    
    QLabel *label_nameprogram = createLabel(this, "ROSA Sync", Qt::AlignCenter, 2);
    label_nameprogram->setFont(createFont("Arial", 12, true, false));

    nameprogramAction = new QWidgetAction(this);
    nameprogramAction->setDefaultWidget(label_nameprogram);

    QLabel *label_lastsynctime = createLabel(this, i18n("      The last synchronization:      "), Qt::AlignCenter, 5);
    label_lastsynctime->setFont(createFont("Arial", 10, false, true));

    lastsynctimeAction = new QWidgetAction(this);
    lastsynctimeAction->setDefaultWidget(label_lastsynctime);

    syncrAction = new QAction(i18n("Syncronize"), this);
    syncrAction->setEnabled(false);
    connect(syncrAction, SIGNAL(activated()), this, SLOT(syncronizeStartStop()) );

    opensettingsAction = new QAction(i18n("Settings"), this);
    connect(opensettingsAction, SIGNAL(activated()), this, SLOT(openSettings()));   
}

void PSyncNotify::closeApp()
{
    m_app->setQuitOnLastWindowClosed(true);
    close();
}

void PSyncNotify::createTrayIcon()
{
    trayIconMenu = new QMenu(this);    

    trayIconMenu->addAction(nameprogramAction);
    trayIconMenu->addSeparator();
    trayIconMenu->addAction(lastsynctimeAction);
    trayIconMenu->addAction(syncrAction);
    trayIconMenu->addAction(opensettingsAction);    

    trayIcon = new QSystemTrayIcon(this);
    trayIcon->setContextMenu(trayIconMenu);

    trayIcon->setIcon(*m_icon);
    connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
}

QLabel* PSyncNotify::createLabel(QWidget *widget, const QString &text, Qt::AlignmentFlag aligment, int margin)
{
    QLabel *label = new QLabel(widget);
    label->setText(text);
    label->setAlignment(aligment);
    label->setMargin(margin);

    return label;
}

QFont PSyncNotify::createFont(const QString &name, int size, bool is_bold, bool is_italic)
{
    QFont font(name, size);
    font.setBold(is_bold);
    font.setItalic(is_italic);

    return font;
}

void PSyncNotify::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
    if (reason == QSystemTrayIcon::Context)
    {
        QDesktopWidget dw;
        const QRect rect = dw.availableGeometry(trayIconMenu);
        int _top = trayIconMenu->geometry().top();
        int _bottom = trayIconMenu->geometry().bottom();
        int abottom = rect.bottom();
        trayIconMenu->setGeometry(trayIconMenu->geometry().x(), _top - (_bottom - abottom), trayIconMenu->geometry().width(), trayIconMenu->geometry().height());
    }
}

void PSyncNotify::receiveNotification()
{
    IPCPacket *m_pkt = NULL;
    int total_bytes = m_us->bytesAvailable();
    if (m_pkt)
        return;

    m_pkt = new IPCPacket();
    m_us->readDatagram((char*)m_pkt->readBuffer(total_bytes), total_bytes);

    if (m_pkt->IsValid())
    {               
        switch (m_pkt->getType())
        {
            case NOTIFICATION_MESSAGE:                
                KNotification::event(KNotification::Notification, QString(TITLE), QString::fromLocal8Bit((const char*)m_pkt->getData(), m_pkt->getDataLen()), QPixmap(), trayIcon->contextMenu());
                break;
            case SYNCRONIZATION_STATUS:
                qDebug("receiveNotification");
                qDebug("data = %s, len = %d", m_pkt->getData(), m_pkt->getDataLen());
                setState(m_pkt->getData(), m_pkt->getDataLen());                
                break;
            case CURRENT_SYNC_STATUS:
                checkStateStatus();
                break;
            case SYNCRONIZE_IS:
                setSyncronizeEnable(m_pkt->getData(), m_pkt->getDataLen());
                break;
            default:
                qDebug("Unknown type of message. The number of message is %d", (int)m_pkt->getType());
        }
    }

    if (m_pkt)
        delete m_pkt;
}

void PSyncNotify::syncronizeStartStop()
{
    const char *data;
    QString button_text;

    qDebug("m_sync_is_running = %d, m_request_lastsync_from_sync = %d", m_sync_is_running, m_request_lastsync_from_sync);
    if (m_sync_is_running)
    {
        data = STOP_SYNCRONIZATION;
        button_text = i18n("Syncronize");

        m_request_lastsync_from_sync = true;
        setLastSyncNote();
    }
    else
    {
        data = START_SYNCRONIZATION;
        button_text = i18n("Stop syncronize");
    }

    IPCPacket* pkt = IPCPacket::Create(SYNCRONIZATION_STATUS, (char*)NOTIFY_PROGRAM_NAME, (unsigned char*)data, (unsigned long)strlen(data));

    QString receiver_address(QDir::homePath() + (const char*)PROGRAM_CATALOG + (const char*)SYNCRONIZE_PROGRAM_NAME);
    if (m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), receiver_address.toLocal8Bit().data()) < 0)
    {
        qDebug("Error of writeDatagram");
        QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);

        if (pkt)
            delete pkt;

        return;
    }

    receiver_address = QDir::homePath() + (const char*)PROGRAM_CATALOG + (const char*)CONFIG_PROGRAM_NAME;
    if (m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), receiver_address.toLocal8Bit()) < 0)
    {
        if (m_process->state() == QProcess::Running)
        {
            qDebug("Error of writeDatagram");
            QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
        }
    }

    syncrAction->setText(button_text);
    m_sync_is_running = !m_sync_is_running;
    setIconSync();

    if (pkt)
        delete pkt;
}

void PSyncNotify::setState(unsigned char *comm, unsigned long len)
{    
    unsigned char data[len + 1];
    memcpy(data, comm, len);
    data[len] = '\0';

    if (strcmp(START_SYNCRONIZATION, (const char*)data) == 0)
    {
        qDebug("1 setState, comm   = %s", comm);
        syncrAction->setText(i18n("Stop syncronize"));
        m_sync_is_running = true;
    }
    else
    {
        qDebug("2 setState, comm   = %s", comm);
        syncrAction->setText(i18n("Syncronize"));
        m_sync_is_running = false;

        check_request_lastsync_from_config();
    }
    setIconSync();
}

void PSyncNotify::initSocket()
{
    m_us = new UnixSocket();
    if (m_us->init())
    {
        qDebug("sockfd = %d", m_us->getSockfd());
        QString self_address(QDir::homePath() + (const char*)PROGRAM_CATALOG + (const char*)NOTIFY_PROGRAM_NAME);
        QDir dir(QDir::homePath() + (const char*)PROGRAM_CATALOG);
        if (!dir.exists())
            dir.mkdir(QDir::homePath() + (const char*)PROGRAM_CATALOG);

        if (m_us->bind(self_address.toLocal8Bit().data()))
        {
            m_sn = new QSocketNotifier(m_us->getSockfd(), QSocketNotifier::Read);
            connect(m_sn , SIGNAL(activated(int )), this, SLOT(receiveNotification()));
        }
        else
        {
            qDebug("Error of local socket bind");
            QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
        }
    }
    else
    {        
        qDebug("Error of local socket init");
        QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
    }

    m_sync_is_running = false;
}

void PSyncNotify::openSettings()
{
    switch (m_process->state())
    {
        case QProcess::NotRunning:
            saveSettings();            
            startSettingsProcess();            
            break;
        case QProcess::Running:
            sendCommandToSettingsWindow(ACTIVATE_COMMAND);                            
            break;
        default:
            qDebug("state of settings process is %d", m_process->state());
    }
}

void PSyncNotify::initProcessManager()
{
    m_process = new KProcess();
    *m_process << QString(SETTINGS_PROGRAM_PATH);
}

void PSyncNotify::startSettingsProcess()
{            
    m_process->start();
}

void PSyncNotify::sendCommandToSettingsWindow(int type_command)
{
    const char *data = (const char*)"1";    //Ничего незначащие данные, в данном случае смысловую нагрузку несёт только тип сообщения
    IPCPacket* pkt = IPCPacket::Create(type_command, (char*)NOTIFY_PROGRAM_NAME, (unsigned char*)data, (unsigned long)strlen(data));

    QString receiver_address(QDir::homePath() + (const char*)PROGRAM_CATALOG + (const char*)CONFIG_PROGRAM_NAME);
    if (m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), receiver_address.toLocal8Bit().data()) < 0)
    {
        if (m_process->state() == QProcess::Running)
        {
            qDebug("Error of writeDatagram");
            QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
        }
    }

    if (pkt)
        delete pkt;
}

void PSyncNotify::readSettings()
{    
    int size_value = 512;
    char value[size_value];

    if (get_param_str(m_pathname_file_config.toLocal8Bit().data(), CONFIG_FILE_LASTSYNC, value, size_value) == LIBCFG_SUCCESS)    //m_sync_settings->read(QString(LAST_TIME_SYNCRONIZATION_SETTING), variant)
        if (value[0] != ' ')
        {
            setLastSyncNote(QVariant(value).toDateTime());
            return;
        }


    QLabel *label = (QLabel*)lastsynctimeAction->defaultWidget();
    label->setText(i18n("      The last synchronization: ") + i18n("unknown"));
}

void PSyncNotify::saveSettings()
{    
    if (m_request_lastsync_from_sync)
    {
        if (m_datetime_lastsync.isNull())
        {
            if (set_param_str(m_pathname_file_config.toLocal8Bit().data(), CONFIG_FILE_LASTSYNC, (char*)" ") != LIBCFG_SUCCESS)   //!m_sync_settings->save(QString(LAST_TIME_SYNCRONIZATION_SETTING), QVariant(m_datetime_lastsync))
                QMessageBox::critical(this, QString("Sync Client"), i18n("Error saving settings"), QMessageBox::Ok);
        }
        else
        {
            if (set_param_str(m_pathname_file_config.toLocal8Bit().data(), CONFIG_FILE_LASTSYNC, QVariant(m_datetime_lastsync).toString().toLocal8Bit().data()) != LIBCFG_SUCCESS)   //!m_sync_settings->save(QString(LAST_TIME_SYNCRONIZATION_SETTING), QVariant(m_datetime_lastsync))
                QMessageBox::critical(this, QString("Sync Client"), i18n("Error saving settings"), QMessageBox::Ok);
        }
    }
}

void PSyncNotify::syncSettingsInstance()
{
    m_pathname_file_config = QString("%1%2%3").arg(QDir::homePath()).arg(PROGRAM_CATALOG).arg(SETTINGS_CATALOG);   
}

void PSyncNotify::setLastSyncNote(const QDateTime &lastsync)
{    
    QLabel *label = (QLabel*)lastsynctimeAction->defaultWidget();
    KLocale *aLocale = KGlobal::locale();

    if (lastsync.isNull())
        m_datetime_lastsync = QDateTime::currentDateTime();
    else
        m_datetime_lastsync = lastsync;

    label->setText(i18n("      The last synchronization: ") + aLocale->formatDateTime(m_datetime_lastsync, KLocale::FancyShortDate).toLower());
}

void PSyncNotify::checkStateStatus()
{
    const char *data;

    if (m_sync_is_running)
        data = START_SYNCRONIZATION;
    else
        data = STOP_SYNCRONIZATION;

    IPCPacket* pkt = IPCPacket::Create(SYNCRONIZATION_STATUS, (char*)NOTIFY_PROGRAM_NAME, (unsigned char*)data, (unsigned long)strlen(data));

    QString receiver_address = QDir::homePath() + (const char*)PROGRAM_CATALOG + (const char*)CONFIG_PROGRAM_NAME;
    if (m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), receiver_address.toLocal8Bit()) < 0)
    {        
        if (m_process->state() == QProcess::Running)
        {
            qDebug("Error of writeDatagram");
            QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
        }
    }

    if (pkt)
        delete pkt;
}

void PSyncNotify::initSyncranizationState()
{
    const char *data = (const char*)"1";    //Ничего незначащие данные, в данном случае смысловую нагрузку несёт только тип сообщения
    IPCPacket* pkt = IPCPacket::Create(CURRENT_SYNC_STATUS, (char*)NOTIFY_PROGRAM_NAME, (unsigned char*)data, (unsigned long)strlen(data));

    QString receiver_address(QDir::homePath() + (const char*)PROGRAM_CATALOG + (const char*)SYNCRONIZE_PROGRAM_NAME);
    if (m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), receiver_address.toLocal8Bit().data()) < 0)
    {
        qDebug("Error of writeDatagram");
        QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);

        if (pkt)
            delete pkt;

        m_request_lastsync_from_sync = true;
        return;
    }

    if (pkt)
        delete pkt;

    m_request_lastsync_from_sync = false;
    qDebug("initSyncranizationState");
}

void PSyncNotify::check_request_lastsync_from_config()
{
    if (m_request_lastsync_from_sync)
        setLastSyncNote();
    else
        m_request_lastsync_from_sync = true;
}

void PSyncNotify::setSyncronizeEnable(unsigned char* comm, unsigned long len)
{
    unsigned char data[len + 1];
    memcpy(data, comm, len);
    data[len] = '\0';

    if (strcmp("1", (const char*)data) == 0)
    {
        qDebug("Sync yes!!!, comm   = %s", comm);
        syncrAction->setEnabled(true);
        trayIcon->setIcon(QIcon(":synchronize.png"));
    }
    else
    {
        qDebug("Sync no!!!, comm   = %s", comm);
        syncrAction->setEnabled(false);
        trayIcon->setIcon(QIcon(":red_status.png"));
    }
}

void PSyncNotify::setIconSync()
{
    if (syncrAction->text() == i18n("Syncronize"))
    {
        qDebug("syncronize.png");
        if (syncrAction->isEnabled())
        {
            m_gif->stop();
            trayIcon->setIcon(QIcon(":synchronize.png"));
        }
    }
    else
    {
        qDebug("animation.gif");
        m_gif->start();
    }
}

void PSyncNotify::updateIcon()
{
    trayIcon->setIcon(m_gif->currentPixmap());
}

/* //Code for test of receiving data from stream
 ______________________________________________________________________________________________________________________
    const char *data = (const char*)"Process of syncronozation is started";
    IPCPacket* pkt = IPCPacket::Create(1, (char*)"/tmp/psyncclient/psyncclientd", (unsigned char*)data, strlen(data));

    int total_bytes= pkt->getSize();
    int readed = 0; //m_sync_settings = new PSyncSettings(pathname_file_config,  QSettings::IniFormat);

    QFile socket_send("socket.txt");
    QFile socket_recv("socket.txt");

    IPCPacket *m_pkt = NULL;

    int i = 0;

    while (total_bytes > 0)
    {
        if (!socket_send.open(QIODevice::WriteOnly | QIODevice::Truncate))
            return;

        if (socket_send.write((const char*)pkt->getBuffer() + i++, 1) < 0)
            return;

        socket_send.close();

        if (m_pkt == NULL)
            m_pkt = new IPCPacket();

        if (!socket_recv.open(QIODevice::ReadOnly))
            return;

        if ((readed = socket_recv.read((char*)m_pkt->readBuffer(), m_pkt->readNeeded()) ) < 0)
            return;

        socket_recv.close();

        m_pkt->readResult(readed);
        total_bytes -= readed;

        if (m_pkt->readNeeded() == 0)
        {
            unsigned long size = m_pkt->getSize();
            unsigned long type = m_pkt->getType();
            char* sender = m_pkt->getSender();
            unsigned char* data = m_pkt->getData();
            qDebug("size = %d, type = %d, sender = %s, data = %s", (int)size, (int)type, sender, data);
            delete m_pkt;
        }
    }

    delete pkt; */
