#define HTTP_PORT 	443
#include <string>
#include <fstream>
#include <sstream>
#include <cstdlib>
#include <errno.h>
#include <stdio.h>
#include <memory.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <map>
#include <libcfg.h>
#include "WebDAV.h"
#include "http_hdr.h"
#include "tinyxml.h"


static void NamespaceClear(TiXmlNode* doc)
{
	if(!doc)
		return;
	string tmp;
	TiXmlNode* pChild;
	tmp=doc->ValueStr();
	if(tmp.find(":")!=string::npos){
		doc->SetValue(tmp.substr(tmp.find(":")+1,string::npos));
	}
	for ( pChild = doc->FirstChild(); pChild != 0; pChild = pChild->NextSibling())
		NamespaceClear(pChild);

}
//static 
using namespace std;

WebDav::WebDav()
{
}
     

WebDav::~WebDav()
{
}

//Приватные члены)
//Файлы
time_t WebDav::GetATime(string path)
{
	struct stat st;
	int ret;
	ret=stat(path.c_str(),&st);
	if(ret<0)
		return ret;
	return st.st_atime;
}

string WebDav::getFileETag(file_s f)
{
	return propfind(f.uname,string(),"getetag");
}
//Команды Вебдав
/** Получение одного свойства файла
 *  если нет lock_token - не использовать блокировку, 
 *  если нет prop - отдать просто все свойства в результате
 *  \param r_path - путь для получения свойств
 * 
 */
string WebDav::propfind(string r_path, string lock_token, string prop)
{
	int err;
	//XML
	TiXmlDocument request;
	TiXmlDeclaration* decl=new TiXmlDeclaration("1.0","utf-8","");
	TiXmlElement* lp= new TiXmlElement("propfind");
	TiXmlElement* la= new TiXmlElement("allprop");
	lp->SetAttribute("xmlns","DAV:");
	lp->LinkEndChild(la);
	request.LinkEndChild(decl);
	request.LinkEndChild(lp);

	TiXmlPrinter print;
	print.SetIndent(" ");
	request.Accept(&print);
	//\XML
	http_hdr hdr(http_hdr::CM_PFIND, r_path);
	hdr.values["Depth"]="1";
	hdr.add_lt(lock_token);
//	hdr.add_body(print.Str().c_str(), print.Str().size());
	hdr.process_request();
	last_error=err=hdr.status;
	if(err!=207)
		return string();
	if(prop.empty())
		return string(hdr.rdata,hdr.cl);
	//Ищем свойство	
	TiXmlDocument ret_request;
	ret_request.Parse(hdr.rdata);
	NamespaceClear(ret_request.RootElement());
	TiXmlHandle handle(&ret_request);
	TiXmlNode* child=handle.FirstChild("multistatus").FirstChild("response").FirstChild("propstat").FirstChild("prop").FirstChild(prop).FirstChild().ToNode();
	if(!child)
		return string();
	return child->ValueStr();
}


int WebDav::setprop(string r_path, string prop, string val, string lock_token)
{
	int err;
	//XML
	TiXmlDocument request;
	TiXmlDeclaration* decl=new TiXmlDeclaration("1.0","utf-8","");
	TiXmlElement* lp= new TiXmlElement("propertyupdate");
	lp->SetAttribute("xmlns","DAV:");
	TiXmlElement* ls= new TiXmlElement("set");
	TiXmlElement* lp1= new TiXmlElement("prop");
	TiXmlElement* lp2= new TiXmlElement(prop);
	TiXmlText* lt= new TiXmlText(val);
	request.LinkEndChild(decl);
	lp2->LinkEndChild(lt);
	lp1->LinkEndChild(lp2);
	ls->LinkEndChild(lp1);
	lp->LinkEndChild(ls);
	request.LinkEndChild(lp);


	TiXmlPrinter print;
	print.SetIndent(" ");
	request.Accept(&print);
	//\XML
	
	http_hdr hdr(http_hdr::CM_PPATCH, r_path);
	hdr.add_lt(lock_token);
	hdr.add_body(print.Str().c_str(), print.Str().size());

	hdr.process_request();
	last_error=err=hdr.status;
	
	if(err<200 || err>299)
		return -1;
	//TODO:Разбор ответа сервера на предмет зафейленых статусов
	return 0;
}

int WebDav::put(string filename,string r_path, string lockt)
{
	int err;
	struct stat st;
	ifstream fd(filename.c_str());	

//	fd.exceptions(ios_base::badbit|ios_base::eofbit|ios_base::failbit);
	if ( !fd.is_open() ) 
		return -ENOENT;

	if(stat(filename.c_str(), &st)){		//TODO:Тип файла и его права и группы и все такое
		cerr<< "File "<<filename.c_str()<<" stat error"<<endl;
		return errno;
	}

	ostringstream msg;
	msg<<"Загружаем на сервер "<< r_path;
	conn_ctx.ipc->send(ipc_proxy::NOTE_MSG, conn_ctx.pathNotify.c_str(), msg.str().c_str());

	http_hdr hdr(http_hdr::CM_PUT, r_path);
	hdr.add_lt(lockt);
	hdr.add_file_body(&fd, st.st_size);

	hdr.process_request();

	last_error=err=hdr.status;
	return (err<200||err>299)?err:0;
}

int WebDav::get(string filename,string r_path)
{
	int err;
	ofstream fd(filename.c_str(), ios_base::trunc|ios_base::binary);	

	ostringstream msg;
	msg<<"Загружаем с сервера "<< r_path;
	conn_ctx.ipc->send(ipc_proxy::NOTE_MSG, conn_ctx.pathNotify.c_str(), msg.str().c_str());

	if ( !fd.is_open() ) {
		cerr<<"Ошибка записи файла "<< filename<<" Сохраняем в каталог по умолчанию";
		string npath(conn_ctx.defFolder);
		npath+="/";
		npath+=r_path;
		fd.clear();
		fd.open(npath.c_str(), ios_base::trunc|ios_base::binary);

		if ( !fd.is_open() ){
			cerr<<"Ошибка записи файла "<< filename;
			return -ENOENT;
		}
	}
	//TODO:Тип файла и его права и группы и все такое
	http_hdr hdr(http_hdr::CM_GET, r_path);
	hdr.add_file_rbody(&fd);
	if(hdr.process_request()==http_hdr::HH_FAIL){
		fd.close();
		unlink(filename.c_str());
	}
	last_error=err=hdr.status;
	return (err<200||err>299)?err:0;
}
int mkcol(string r_path)
{
	return 0;
}

int WebDav::lock(string r_path, string& lock_token)
{
	int err;
	string req;
	TiXmlDocument request;
	TiXmlDeclaration* decl=new TiXmlDeclaration("1.0","utf-8","");
	TiXmlElement* li= new TiXmlElement("lockinfo");
	TiXmlElement* ls= new TiXmlElement("lockscope");
	TiXmlElement* scope= new TiXmlElement("exclusive");
	TiXmlElement* lt= new TiXmlElement("locktype");
	TiXmlElement* type= new TiXmlElement("write");

	ls->LinkEndChild(scope);
	li->LinkEndChild(ls);
	li->SetAttribute("xmlns","DAV:");
	lt->LinkEndChild(type);
	li->LinkEndChild(lt);
	request.LinkEndChild(decl);
	request.LinkEndChild(li);

	TiXmlPrinter print;
	print.SetIndent(" ");
//	print.SetStreamPrinting();
	request.Accept(&print);
	
	http_hdr hdr(http_hdr::CM_LOCK, r_path);
	hdr.add_lt(lock_token);
	hdr.add_body(print.Str().c_str(), print.Str().size());
	hdr.values["Timeout"]="Second-60";

	hdr.process_request();
	last_error=err=hdr.status;

	if(err<200 || err>299)
		return err;

	TiXmlDocument ret_request;
	ret_request.Parse(hdr.rdata);
	lock_token=hdr.values["Lock-Token"].substr(0,hdr.values["Lock-Token"].length()-1);
	return 0;
}

int WebDav::unlock(string r_path, string lockt)
{
	int err;
	http_hdr hdr(http_hdr::CM_UNLOCK, r_path);
	hdr.add_lt(lockt);
	hdr.process_request();
	last_error=err=hdr.status;
	if(err<200 || err>299)
		return err;
	return 0;
}

int WebDav::Delete(const string path)
{
	int err;
	http_hdr hdr(http_hdr::CM_DELETE, path);
	hdr.process_request();
	last_error=err=hdr.status;
	if(err<200 || err>299)
		return err;
	return 0;
}
int WebDav::Mkcol(const string path)
{/*
	int err;
	string req;
	req=P_MKCOL+path+' '+http_h;
	req+='\n';
	err = send( req.c_str(), req.size());
	if(err<=0)
		return -1;

	char rcvbuf[255];

	err = recv(rcvbuf, 255);
	if(err > 0)
		sscanf(rcvbuf, "HTTP/1.1 %d", &err);
	if(err==200)*/
		return 0;/*
	else
		return err;*/
}

#define NOLOCK
int WebDav::Put(file_s& f)
{
	int ret;
	string lock_token;
	cerr<<"Processing file:"<<f.path<<" ";
#ifndef NOLOCK
	ret=lock(f.uname, lock_token);
	if(ret){
		cerr<<"LOCK error:"<<ret<<endl;
		return -1;
	}
#endif
	ret=put(f.path,f.uname, lock_token);
	if(!ret){
		ret=setprop(f.uname, "clientpath", f.path, lock_token);
		f.eTag=propfind(f.uname,lock_token,"getetag"); 
	}
#ifndef NOLOCK
	ret=unlock(f.uname, lock_token);
	if(ret){
		cerr<<"UNLOCK error:"<<ret<<endl;
		return -1;
	}
#endif
	f.sync_time=GetATime(f.path);
	return 0;
}

int WebDav::Get(file_s& f)
{
	int ret=get(f.path,f.uname);
	if(!ret){
		f.sync_time=GetATime(f.path);
		f.eTag=propfind(f.uname,string(),"getetag");
	}
	return ret;
}

vector<file_s> WebDav::GetServerList()
{
	vector<file_s> ret;
	string allprop=propfind(string(),string(),string());
	//XML
	TiXmlDocument ret_request;
	ret_request.Parse(allprop.c_str());
	NamespaceClear(ret_request.RootElement());
	//TO multistatus
	TiXmlNode* parent=ret_request.FirstChild("multistatus");
	if(parent){
		TiXmlNode* child=0;	
		while((child = parent->IterateChildren("response",child))!=NULL){			//TODO:Возможно контролировать по response/propstat/status?
			file_s f;
			TiXmlHandle i(child);
			TiXmlElement* element;
			//Путь к файлу на сервере
			element = i.FirstChild("href").ToElement();
			if(!element)
				continue;
			f.uname = element->GetText();
			f.uname = f.uname.substr(f.uname.rfind("/")+1);				//Обрезаем до имени файла	
			//Путь на клиенте
			element = i.FirstChild("propstat").FirstChild("prop").FirstChild("clientpath").ToElement();
			if(!element)
				continue;
			f.path = element->GetText();
			//Етаг
			element = i.FirstChild("propstat").FirstChild("prop").FirstChild("getetag").ToElement();
			if(!element)
				continue;
			f.eTag = element->GetText();
			f.flags=0;
			f.sync_time=0;
			ret.push_back(f);
		}
	}
	
	return 	ret;
}

int WebDav::TryAuth()
{
	long long max, cur;
	if(!GetQuote(max, cur)){
		set_param_ull((conn_ctx.home+"info/").c_str(),"quota_total",max);
		set_param_ull((conn_ctx.home+"info/").c_str(),"quota_used",cur);
	}
	return last_error;
}

int WebDav::GetQuote(long long& max, long long& cur)
{
	string allprop=propfind(string(),string(),string());

	//XML
	TiXmlDocument ret_request;
	ret_request.Parse(allprop.c_str());
	NamespaceClear(ret_request.RootElement());
	TiXmlHandle handle(&ret_request);
	TiXmlElement* child=handle.FirstChild("multistatus").FirstChild("response").FirstChild("propstat").FirstChild("prop").FirstChild("quota-available-bytes").ToElement();
	if(!child)
		return last_error;
	max =atoll(child->GetText());
	child=handle.FirstChild("multistatus").FirstChild("response").FirstChild("propstat").FirstChild("prop").FirstChild("quota-used-bytes").ToElement();
	if(!child)
		return last_error;
	cur =atoll(child->GetText());
	return 0;
}
