#include <fstream>
#include <errno.h>
#include <time.h>
#include <stdlib.h>
#include <pthread.h>
#include "filesdb.h"
#include "uuid/uuid.h"

ostream& operator<<(ostream& s,file_s f)
{
	unsigned sz=f.uname.size();
	s.write((char*)&sz,sizeof(sz));
	s.write(f.uname.c_str(),f.uname.size());
	sz=f.path.size();
	s.write((char*)&sz,sizeof(sz));
	s.write(f.path.c_str(),f.path.size());
	s.write((char*)&f.sync_time,sizeof(f.sync_time));
	sz=f.eTag.size();
	s.write((char*)&sz,sizeof(sz));
	s.write(f.eTag.c_str(),sz);
	s.write((char*)&f.flags,sizeof(f.flags));
	return s;
}

istream& operator>>(istream& s,file_s &f)
{
	unsigned sz;
	char* buf;

	s.exceptions(ios_base::badbit|ios_base::eofbit|ios_base::failbit);
	s.read((char*)&sz,sizeof(sz));
	buf=new char[sz];
	s.read(buf,sz);
	f.uname=string(buf,sz);
	delete []buf;

	s.read((char*)&sz,sizeof(sz));
	buf=new char[sz];
	s.read(buf,sz);
	f.path=string(buf,sz);
	delete []buf;

	s.read((char*)&f.sync_time,sizeof(f.sync_time));	

	s.read((char*)&sz,sizeof(sz));
	buf=new char[sz];
	s.read(buf,sz);
	f.eTag=string(buf,sz);
	delete []buf;
	s.read((char*)&f.flags,sizeof(f.flags));

	return s;
}



filedb::filedb(string fpath):path(fpath)
{
	pthread_mutex_init(&m_lock,0);
}

filedb::filedb()
{
	pthread_mutex_init(&m_lock,0);
}

filedb::~filedb()
{
	pthread_mutex_destroy(&m_lock);
}

void filedb::dbset(string fpath)
{
	path=fpath;
}

int filedb::lock()
{
	return pthread_mutex_lock(&m_lock);
}

int filedb::unlock()
{
	return pthread_mutex_unlock(&m_lock);
}
int filedb::load()
{
	file_s buf;
	ifstream f(path.c_str());
	if(!f)
		return -EACCES;
	m_db.clear();
	try{
		while(f>>buf)
			m_db.push_back(buf);
	}
	catch(ios_base::failure){
	}
	f.close();
	return 0;
}

int filedb::save()
{
	struct file_s buf;
	ofstream f(path.c_str(),ios_base::out|ios_base::trunc);
	if(!f)
		return -EACCES;
	for(vector<file_s>::iterator i=m_db.begin(); i!=m_db.end(); i++)
		f<<*i;
	f.close();
	return 0;
}
#ifdef USE_UUID
static string genID()
{
	uuid_t id;
	char c_id[37];
	uuid_generate(id);
	uuid_unparse(id, c_id);
	return string(c_id);
}
#endif
int filedb::insert(string fpath)
{
	for(vector<file_s>::iterator i=m_db.begin(); i!=m_db.end(); i++)
		if(i->path==fpath)
			return -EEXIST;
	lock();
	file_s f;
	f.path=fpath;
#ifdef USE_UUID
	f.uname=genID(); //UUID
#else
	f.uname=f.path.substr(f.path.rfind("/")+1);
#endif
	f.flags=0;
	f.sync_time=0;
	m_db.push_back(f);
	save();
	unlock();
	return 0;
}

int filedb::remove(string fpath)
{
	vector<file_s>::iterator i;
	lock();
	for(i=m_db.begin(); i!=m_db.end(); i++)
		if(i->path==fpath)
			break;
	if(i==m_db.end()){
		unlock();
		return -1;
	}
	i->flags|=FL_DELETE;
	save();
	unlock();
	return 0;
}

vector<file_s>& filedb::getFileVec() 
{
	return m_db;
//	vector<file_s> ret(m_db);
//	return ret;
}
