#include <string>
#include <map>
#include <sstream>
#include <iostream>
#include <stdlib.h>
#include <memory.h>
#include <netdb.h>
#include <sys/socket.h>
#include <signal.h>
#include <errno.h>
#include <sys/ioctl.h>
#include "main.h"

#include "base64.h"
#include "http_hdr.h"

//#define DEBUG
using namespace std;

string str_http_cmd[]=
{
	"MKCOL",
	"PUT",
	"PROPPATCH",
	"GET",
	"LOCK",
	"UNLOCK",
	"PROPFIND",
	"DELETE",
};


http_hdr::~http_hdr()
{
	if(data)
		delete data;
	if(rdata)
		delete rdata;
	close(sock);
	//SSL
	if(ssl_ctx)
		SSL_CTX_free(ssl_ctx);
}

http_hdr::http_hdr(http_command command, string filename):fname(filename)
{
	ostringstream hdr;
	hdr<<str_http_cmd[command]<<" "<<conn_ctx.path<<filename<<" HTTP/1.1"<<endl;
	header=hdr.str();
	values["Host"]=conn_ctx.host;
	if(!conn_ctx.creds.empty()){
		values["Authorization"]="Basic "+base64_encode(conn_ctx.creds);
	}

	sock=0;
	ssl_ctx=NULL;
	ssl=NULL;
	mode=CT_NORMAL;

	data=NULL;
	data_size=0;
	rdata=NULL;
	conn_state=WD_DISCONNECTED;
}

void http_hdr::add_body(const void* buf, size_t len)
{
	data=new char[len];
	memcpy(data, buf, len);
	data_size=len;
	ostringstream _cl;
	_cl<<len;
	values["Content-Length"]=_cl.str();
}

void http_hdr::add_lt(std::string lock_token)
{
	if(!lock_token.empty()){
		values["If"]="(";
		values["If"]+=lock_token+")";
	}
}

static void sigpipe_handle(int s)
{
	return;
}

void http_hdr::connect()
{
	if(conn_state==WD_CONNECTED)
		return;
	sockaddr_in name;
	name.sin_family = AF_INET;
	name.sin_port   = htons( conn_ctx.port );
	hostent* hoste = gethostbyname(conn_ctx.host.c_str());
	if (hoste == NULL) {
		throw "cannot find host to connect";
	}
	memcpy(&name.sin_addr,hoste->h_addr, hoste->h_length);
	if(conn_state==WD_EPIPE){
		//TODO: sess=SSL_get_session(ssl)
		SSL_shutdown(ssl);
		SSL_free(ssl);
		close(sock);

		conn_state=WD_DISCONNECTED;
	}

	sock=socket(AF_INET, SOCK_STREAM, 0);
	int err = ::connect(sock, (sockaddr*)&name, sizeof(sockaddr_in));
	if(err){
		perror("connect error");
		throw "connect error";
	}
	if(conn_ctx.ssl){
		if(!ssl){
			bio_err=0;
			SSL_METHOD *meth;

			if(!bio_err){
				SSL_library_init();
				SSL_load_error_strings();
				
				bio_err=BIO_new_fp(stderr,BIO_NOCLOSE);
			}
			
			/* Set up a SIGPIPE handler */
			signal(SIGPIPE,sigpipe_handle);
			
			meth=(SSL_METHOD*)SSLv23_method();
			ssl_ctx=SSL_CTX_new(meth);
		}

		ssl=SSL_new(ssl_ctx);
		sbio=BIO_new_socket(sock,BIO_NOCLOSE);
		SSL_set_bio(ssl,sbio,sbio);

		if(SSL_connect(ssl)<=0)
			throw "SSL connect error";
		//TODO:	check_cert(ssl,host.c_str());
	}

	conn_state=WD_CONNECTED;
}
int http_hdr::_send(const void* buf, size_t len)
{
	int ret=HH_FAIL;
	int r;

	if(conn_ctx.ssl){
		if(conn_state!=WD_CONNECTED)
			connect();
		r=SSL_write(ssl, buf, len);
		  
		switch(SSL_get_error(ssl,r)){
			/* We wrote something*/
			case SSL_ERROR_NONE:			
				break;
			case SSL_ERROR_ZERO_RETURN:
				conn_state=WD_EPIPE;
				ret=HH_RETRY;
				break;				
			case SSL_ERROR_SYSCALL:
				if(r==-1 && errno == EPIPE){
					conn_state=WD_EPIPE;
					ret=HH_RETRY;
					break;
				}
		  	default:
				cerr<<"Unrecoveroble SSL write error :"<< SSL_get_error(ssl,r)<<":"<<r<<":"<<errno<<endl;		
		}
	} else {
		r=::send(sock, buf, len, MSG_NOSIGNAL);
		if(r==-1 &&
			(errno==ENOTCONN || errno==EPIPE)){
			conn_state=WD_EPIPE;
			ret=HH_RETRY;
		}
	}
	if(r > 0 && (unsigned)r==len)
		ret=HH_OK;
#ifdef DEBUG
	if(r>0)
		cerr<<endl<<"\033[01;34m"<<string((char*)buf,r)<<"\033[00m"<<endl;
#endif
	return ret;
}
int http_hdr::_recv(void* buf, size_t len) 
{
	int r=HH_FAIL;
	while(len){
		if(conn_ctx.ssl){
			r=SSL_read(ssl,buf,len);
			switch(SSL_get_error(ssl,r)){
				case SSL_ERROR_NONE:
			      		break;
				case SSL_ERROR_ZERO_RETURN:
					conn_state=WD_EPIPE;
					return HH_RETRY;
				case SSL_ERROR_WANT_READ:
			      		throw "read_blocked=1";
			      		break;
				case SSL_ERROR_WANT_WRITE:
					throw "read_blocked_on_write=1";
			      		break;
				case SSL_ERROR_SYSCALL:
					if(r==-1 && errno == EPIPE){
						conn_state=WD_EPIPE;
						return HH_RETRY;
					}

			    	default:
					cerr<<"Unrecoveroble SSL read error :"<< SSL_get_error(ssl,r)<<":"<<r<<":"<<errno<<endl;		
			}
		} else {
			r=::recv(sock, buf, len, 0);
			if(r==0){
				conn_state=WD_EPIPE;
				return HH_RETRY;
			}
		}
#ifdef DEBUG
		cerr<<endl<<"\033[01;31m" <<string((char*)buf,r)<<"\033[00m"<<endl;
#endif
		len-=r;
		buf=(char*)buf+r;
	}
	return HH_OK;
}

http_hdr::proto_ret http_hdr::send()
{
	http_hdr::connect();
	int ret;

	//Собираем полностью всё
	string head=toString();
	if(mode != CT_FILEIN){ //НЕ ФАЙЛ
		char buf[data_size+head.length()];
		memcpy(buf, head.c_str(), head.length());
		if(data_size)
			memcpy(buf+head.length(), data, data_size);
		//Посылаем
		ret=_send(buf, data_size+head.length());
		if(ret) return (proto_ret)ret;
	}else {				//CT_FILEIN
		ret=_send(head.c_str(),head.length());	
		if(ret) return (proto_ret)ret;
		char buf[1024*512]; 	//512K
//		size_t transmitted=0;
		do
		{
			ifd->read(buf,1024*512);
			ret =_send(buf,ifd->gcount());
			if(ret) {
				ifd->seekg(0);
				return (proto_ret)ret;
			}
			//IPC
/*			transmitted+=ifd->gcount();
			ostringstream msg;
			msg<<fname<<":"<<transmitted<<":"<<values["Content-Length"];
			conn_ctx.ipc->send(ipc_proxy::NOTE_MSG, conn_ctx.pathNotify.c_str(), msg.str().c_str());*/
			//\IPC
		}
		while( !ifd->eof());
	}

	ret = recv();
	if(ret==HH_OK){			//Обработка 301 MOVED
		if(status==301) {	//Переписывает сервер и AGAIN
			conn_ctx.host=ivalues["Location"];
			values["Host"]=conn_ctx.host;
			string::size_type i=conn_ctx.host.find("//")+2;
			conn_ctx.host=conn_ctx.host.substr(i, conn_ctx.host.find("/",i));
			conn_state=WD_EPIPE;
			return HH_RETRY;			
		}if(status==401){		//Если 401 - мочим логин и пароль
			unlink((conn_ctx.home+"cfg/user/login").c_str());
			unlink((conn_ctx.home+"cfg/user/password").c_str());
		}
	}
	return (proto_ret)ret;
}

int  http_hdr::pending()
{
	fd_set fds;
	struct timeval tv;
	int r;
	FD_ZERO (&fds);
	FD_SET (sock, &fds);
	/* Timeout. */
	tv.tv_sec = 5;
	tv.tv_usec = 0;

	r = select (sock+1, &fds, NULL, NULL, &tv);

	if (-1 == r) {
		if (EINTR == errno)
			throw "Socket select error";
	}
//	if (0 == r) 		//Убрано Для контроля за состоянием соединения
//		return 0;

	if(ssl){
		int r=SSL_peek(ssl,&r,1);
		switch(SSL_get_error(ssl,r)){
			case SSL_ERROR_NONE:
				break;
			case SSL_ERROR_ZERO_RETURN:
				conn_state=WD_EPIPE;
				return -HH_RETRY;
			case SSL_ERROR_WANT_READ:
				throw "read_blocked=1";
				break;
			case SSL_ERROR_WANT_WRITE:
				throw "read_blocked_on_write=1";
				break;
			case SSL_ERROR_SYSCALL:
				if(r==-1 && errno == EPIPE){
					conn_state=WD_EPIPE;
					return -HH_RETRY;
					break;
				}
			default:
				cerr<<"Unrecoveroble SSL read error :"<< SSL_get_error(ssl,r)<<":"<<r<<":"<<errno<<endl;		
				return -HH_FAIL;
		}
		return SSL_pending(ssl);
	}else{
		int pending;
		if(ioctl(sock,FIONREAD,&pending)==0)		//TODO: проверка на обрыв и ретрай
			return pending; 
		else return -HH_RETRY;
	}

}

void http_hdr::parse_rcvd(string _s)
{
	status=-1;
	if(rdata){
		delete rdata;
		rdata=NULL;
	}
	ivalues.clear();
	end=_s.find("\r\n\r\n");
	istringstream s(_s.substr(0, end));
	if(end!=string::npos)
		end+=4;
	string line;
	getline(s, line);
	string::size_type i;
	i=line.find("HTTP/1.1 ");
	if(i==string::npos)
		return;
	status=atoi(line.substr(i+9,string::npos).c_str());
	while(getline(s,line)){
		i=line.find(":");
		if(i==string::npos)
			return;
		ivalues[line.substr(0,i)]=line.substr(i+2);
	}
	more_data_size=cl=atoi(ivalues["Content-Length"].c_str());
	if(more_data_size>HTTP_HDR_MAX)
		more_data_size=HTTP_HDR_MAX;
	more=more_data_size;
	more_data=rdata=new char[more_data_size];
	memcpy(more_data,_s.c_str()+end,_s.length()-end);
	more_data+=_s.length()-end;
	more-=_s.length()-end;

}

http_hdr::proto_ret http_hdr::recv()
{	
	int ret=HH_RETRY;
	int sz=pending();
	if(sz<0)
		return (proto_ret)-sz;
	char rcvbuf[sz];
	ret = _recv( rcvbuf, sz);
	if(ret) return (proto_ret)ret;
	parse_rcvd(string(rcvbuf,sz));
	if(more)
		_recv(more_data, more);		//получили данные до метра
	//TODO? if(ret) return ret;
	if(mode == CT_FILEOUT){
		int size=cl;
		ofd->write(rdata, more_data_size);
		size-=more_data_size;
		
/*		{
			ostringstream msg;
			msg<<fname<<":"<<cl-size<<":"<<cl;
			conn_ctx.ipc->send(ipc_proxy::NOTE_MSG, conn_ctx.pathNotify.c_str(), msg.str().c_str());
		}*/
		//заливаем файл	
		while(size)
		{
			char buf[1024*512];
			unsigned sz=size>1024*512?1024*512:size;
			if(_recv(buf,sz)!=HH_OK){
				cerr<<"File save error"<<endl;
				cerr<<"File corrupted"<<endl;
				return HH_FAIL;
			}
			ofd->write(buf,sz);
			size-=sz;

/*			ostringstream msg;
			msg<<fname<<":"<<cl-size<<":"<<cl;
			conn_ctx.ipc->send(ipc_proxy::NOTE_MSG, conn_ctx.pathNotify.c_str(), msg.str().c_str());*/
		}
	}
	return HH_OK;
}

string http_hdr::toString()
{
	string ret;
	ret=header;
	for(map<string,string>::const_iterator p=values.begin();p!=values.end();p++)
		ret+=p->first+": "+p->second+"\n";
	ret+="\n";
	return ret;
}

int http_hdr::process_request()
{
	proto_ret err;
	int retries=2; //Один если упало соединение и один для 301 Moved
	do{
		err = send();
		retries--;
	}while(err==HH_RETRY && retries);
	return err;
}
void http_hdr::add_file_body(std::ifstream* fd, size_t size)
{
	ifd=fd;
 	data_size=size;
	mode=CT_FILEIN;
	ostringstream _cl;
	_cl<<size;
	values["Content-Length"]=_cl.str();
}
