#include <sstream>
#include <pthread.h>
#include <sys/poll.h>
#include <errno.h>
#include <iostream>
#include "sync.h"
#include <libcfg.h>
#include "http_hdr.h"

extern http_ctx conn_ctx;
syncronizer::syncronizer(string srv,unsigned short port, string col):WebDav()
{
	is_working=0;
	stopflag=0;
	pthread_spin_init(&lock,0);
	sync_time=0;
	def_synctime=0;
}

syncronizer::~syncronizer()
{
	stop();
	pthread_spin_destroy(&lock);
}

static void* thread(void* arg)
{
	syncronizer* ctx=(syncronizer*) arg;
	ctx->fthread();
	return NULL;
}
void syncronizer::fthread()
{
	while(is_working){
		if(sync_time>0){
			if(!(sync_time--))
				sync_time--;
			poll(0,0,1000);
		}else if(sync_time == 0)
			poll(0,0,100);
		else {
			sync();
			sync_time=def_synctime;
		}
	}
}
int syncronizer::sync()
{

	ipc->send(ipc_proxy::SYNC_START, conn_ctx.pathNotify.c_str());
	ipc->send(ipc_proxy::SYNC_START, conn_ctx.pathConfig.c_str());
	try{
		pthread_spin_lock(&lock);
		db.lock();
		vector <file_s>& files = db.getFileVec();
		vector <file_s> srv_files = WebDav::GetServerList();
		//Сливаем в один
		for(vector<file_s>::iterator j=srv_files.begin();j!= srv_files.end(); j++){
			vector<file_s>::iterator i;
			for(i=files.begin();i!= files.end(); i++){
				if(i->uname == j->uname){
					break;
				}
			}
			if(i==files.end()){
				files.push_back(*j);
				cerr<<"Добавлен с сервера файл "<< j->path<< endl;
			}
		}
			
		/***/
		for(vector<file_s>::iterator i=files.begin();i!= files.end(); i++){
			if(stopflag){
				stopflag=0;
				break;
			}

			string eTag=getFileETag(*i);
			cerr<< "Обработка "<< i->path<<":";
			/*Удаление*/
			if(!eTag.empty() && (i->flags&FL_DELETE)){//Там чтото есть?
				if(WebDav::Delete(i->uname))
					cerr<<" ошибка: "<<last_error<<endl;
				else{
					i->flags|=FL_DELETED;
					cerr<< "удален"<<endl;
				}
				continue;
			}
			/*Теперь синхронизация*/
			if(access(i->path.c_str(),R_OK) && errno==ENOENT){      //      У нас ничего нет
				cerr<< "Нет у нас качаем...";
				WebDav::Get(*i);
				cerr<<endl;
				continue;
			}
			if(eTag.empty()){               //На сервере ничего нет
				WebDav::Put(*i);
				cerr<< "Нет на сервере закачиваем...";
				cerr<<endl;
				continue;
			}
			if(eTag==i->eTag){              //етаги совпадают :Версия актуальная или у нас лучше
				cerr<< "Етаги равны...";
				cerr<<"Наш stime/"<< i->sync_time <<"/"<< GetATime(i->path)<<"/";
				if(i->sync_time < GetATime(i->path)){
					cerr<<"/>заливаем";
					WebDav::Put(*i);
				}
			} else{                           //Версии отличаются - значит на сервере ктото похимичил:)
				cerr<< "Етаги неравны..."<<eTag <<"/"<<i->eTag<<"/..Получаем с сервера";
				WebDav::Get(*i);                //Тащим серверную себе
			}		
			cerr<<endl;
		}
		int deleted;
		do{
			deleted=0;
			for(vector<file_s>::iterator i=files.begin();i!= files.end(); i++){
				if(i->flags&FL_DELETED){
					++deleted;
					files.erase(i);
					break;
				}			
			}
		}while(deleted);
		
		db.save();
		pthread_spin_unlock(&lock);
		db.unlock();
		std::cerr<<"Sync success"<<std::endl;
		long long max, cur;
		if(!GetQuote(max, cur)){
			set_param_ull((conn_ctx.home+"info/").c_str(),"quota_total",max);
			set_param_ull((conn_ctx.home+"info/").c_str(),"quota_used",cur);
		}


	}catch(const char* str)
	{
		std::cerr<<"Sync thread critical error: "<<str<<std::endl;
		db.unlock();
		pthread_spin_unlock(&lock);
		ipc->send(ipc_proxy::SYNC_STOP, conn_ctx.pathNotify.c_str());
		ipc->send(ipc_proxy::SYNC_STOP, conn_ctx.pathConfig.c_str());
		return 0;
	}
	ipc->send(ipc_proxy::SYNC_STOP, conn_ctx.pathNotify.c_str());
	ipc->send(ipc_proxy::SYNC_STOP, conn_ctx.pathConfig.c_str());

	return 0;
}

int syncronizer::dbconnect(string path)
{
	db.dbset(path);
	db.load();
	return 0;
}

int syncronizer::addFile(string path)
{
 	int ret=db.insert(path);
	if(!ret)
		db.save();
	return 0;
}

int syncronizer::removeFile(string path)
{
 	int ret=db.remove(path);
	if(!ret)
		db.save();
	return 0;
}

int syncronizer::checkFile(string path)
{
	vector <file_s>& files = db.getFileVec();
	for(unsigned i=0;i< files.size(); i++)
		if(files[i].path==path)
			return 1;
	return 0;
}

string syncronizer::checkDir(string path)
{
	stringstream ret;
	vector <file_s>& files = db.getFileVec();
	for(unsigned i=0;i< files.size(); i++)
		if( (files[i].path.find(path) == 0) &&
		    (files[i].path.find("/",path.length()+1) == string::npos) )
			ret<<files[i].path<<'\n';
	return ret.str();
}

int syncronizer::is_work()
{
	int ret=pthread_spin_trylock(&lock);
	if(!ret)
	{
		pthread_spin_unlock(&lock);
	}
	return !!ret;
}

void syncronizer::setSyncTime(int sec)
{
	cerr<<"synctime="<<sec<<endl;
	sync_time=sec;
	def_synctime=sec;
}

void syncronizer::syncAll()
{
	sync_time=-1;
}
int syncronizer::start()
{
	if(is_working)
		return -1;
	is_working=1;
	pthread_create(&th, NULL, thread, this);
	return 0;
}
void syncronizer::syncStop()
{
	stopflag=1;
}
void syncronizer::stop()
{
	is_working=0;
	pthread_join(th,NULL);
}
